/************************************************************************
 * Probeprogramm...: empfaenger.c        				*
 * Zweck...........: Empfnger fr verschiedene Sockets         	*
 * Geschichte:                                                          *
 *      1994 : Teil von 'schlauch'                                      *
 * 17.11.2001: Testprogramm fuer Signale und verscheidene Sockets       *
 ************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<sys/un.h>
#include<netinet/in.h>
#include<netdb.h>
#include<signal.h>
#include<errno.h>
#include"messung.h"

int empfaenger(struct sockaddr *,int,int ruhig);
int initaddr(struct sockaddr *partner,int *addrlen, char *addresse);
void cleanupaddr(struct sockaddr *partner);

#define PUFLEN 1024
char puffer[PUFLEN];
char errmsg[PUFLEN];

int v=0;

int beende=0;
void handlesigaction(int sig,siginfo_t *info,void *context)
{
  fprintf(stderr,"Signal %d von Proze %d/User %d empfangen ...\n",
	          sig,info->si_pid,info->si_uid);
  switch(sig)
  {
  case SIGINT:
  case SIGTERM:
	beende=1;
	break;
  } 
}

void installhandler(int sig)
{
  struct sigaction action;
  action.sa_flags = SA_SIGINFO;
  action.sa_sigaction = handlesigaction;
  sigemptyset(&action.sa_mask);
  sigaction(sig,&action,NULL);
}

int main(int argc,char **argv)
{
  char rechner[50];
  int port=0;
  int o;
  struct sockaddr addr;
  int addrlen;
  int ruhig=0;
  while((o=getopt(argc,argv,"vq"))!=-1)
  {
    switch(o)
    {
    case 'v':
      v = 1;
      break;
    case 'q':
      ruhig=1;
      break;
    }
  }

  if(optind == argc)
  {
    /* Empfaenger mit mit Selbstwahlport */
    initaddr(&addr,&addrlen,NULL);
  }
  else if(argc-optind >= 1)
  {
    initaddr(&addr,&addrlen,argv[optind]);
  }
  installhandler(SIGTERM); 
  installhandler(SIGINT); 
  installhandler(SIGUSR1); 
  installhandler(SIGUSR2); 
  empfaenger(&addr,addrlen,ruhig);
}

int initaddr(struct sockaddr *partner,int *addrlen, char *addresse)
{
  char *ep;
  long port;
  if(addresse==NULL) addresse="";
  if(strncmp(addresse,"unix:",5)==0)
  {
    struct sockaddr_un *addr = (struct sockaddr_un *)partner;
    addr->sun_family = AF_UNIX;
    strcpy(addr->sun_path,addresse+5);
    *addrlen = SUN_LEN(addr);
    return 0;
  }
  if(strncmp(addresse,"inet:",5)==0)
  {
    addresse+=5;
  }
  if(1)
  {
    struct sockaddr_in *addr = (struct sockaddr_in *)partner;
    *addrlen=sizeof(struct sockaddr_in);
    port=strtol(addresse,&ep,10);
    /* Binde an Adresse und waehle Portnr. frei falls port == 0; */
    bzero((char *)addr,sizeof(struct sockaddr_in));
    addr->sin_family = AF_INET;
    addr->sin_addr.s_addr = INADDR_ANY;
    addr->sin_port = htons(port);
  }
}

void printaddr(struct sockaddr *addr)
{
  switch(addr->sa_family)
  {
  case AF_INET:
    {
      struct sockaddr_in *maddr = (struct sockaddr_in *)addr;
  
      fprintf(stderr," %s Port: %d\n",inet_ntoa(maddr->sin_addr)
	      ,ntohs(maddr->sin_port));
      return;
    }
  case AF_UNIX:
    {
      struct sockaddr_un *maddr = (struct sockaddr_un *)addr;
      fprintf(stderr,"Unixsocket %s\n",maddr->sun_path);
      return;
    }
  }
}

void printstat(struct timeval *start,struct timeval *stop,long anz)
{
  double zeit = diff(start,stop);
  fprintf(stderr,"%ld bytes in %g s empfangen\n",anz,zeit);
  fprintf(stderr,"Durchsatz %g bytes/s\n",anz/zeit);
}

/********************************************************
 * empfaenger : dient als Server                 	*
 ********************************************************/
int empfaenger(struct sockaddr *maddr,int maddrlen,int ruhig)
{
   int s0,sneu;
   struct sockaddr kundenaddr;
   int kundenaddrlen = sizeof(kundenaddr);
 
   int llang;
   char *ptr;
   int r,rest;   
   struct timeval anfang,ende;
   long receivedbytes=0;
   
   if((s0 = socket(maddr->sa_family,SOCK_STREAM,0)) < 0)
   {   
      sprintf(errmsg,"socket (%s:%d):",__FILE__,__LINE__);
      perror(errmsg);
      exit(1);
   }

   
   if(bind(s0,maddr,maddrlen) < 0)
   {
      sprintf(errmsg,"bind (%s:%d):",__FILE__,__LINE__);
      perror(errmsg);
      exit(1);
   }
      

   if(getsockname(s0,&kundenaddr,&kundenaddrlen)<0)
   {   
      sprintf(errmsg,"getsockname (%s:%d):",__FILE__,__LINE__);
      perror(errmsg);
      exit(1);
   }
   if(kundenaddr.sa_family != AF_UNIX)
   {
     char rechnername[80];
     /* Ermittle Rechner und gewaehlte Protnummer */
     if(gethostname(rechnername,80)<0)
     {
       sprintf(errmsg,"gethostname (%s:%d):",__FILE__,__LINE__);
       perror(errmsg);
       return;
     }
     fprintf(stderr,rechnername);
   }
   printaddr(&kundenaddr);

   listen(s0,1);
   beende=0;
   while((sneu = accept(s0,&kundenaddr,&kundenaddrlen))<0
         && !beende)
   {
      sprintf(errmsg,"accept (%s:%d) Err %d:",__FILE__,__LINE__,errno);
      perror(errmsg);
      switch(errno)
      {
        case EAGAIN:
        case EINTR:
             write(2,"accept: nochmal\n",sizeof("accept: nochmal\n"));
	     errno=0;
             continue;
        default:
          exit(1); 
      }
   }
   if(beende)
   {
     fprintf(stderr,"Beendigung waehrend accept!\n");
     cleanupaddr(maddr);
     close(s0);
     exit(1);
   }
   fprintf(stderr,"Verbindung von ");printaddr(&kundenaddr);

   /* lesen der Ausgabe */
   gettimeofday(&anfang,NULL);
   errno=0;
   while((llang = read(sneu,puffer,PUFLEN-1))!=0 && !beende)
   {
     if(llang<0)
     {
       switch(errno)
       {
         case EAGAIN: 
         case EINTR:
               write(2,"read: nochmal\n",sizeof("read: nochmal\n"));
	       errno=0;
	       continue;
         default:
               break;
       } 
     }
     receivedbytes +=llang;
     puffer[llang]='\0';
     rest = llang;
     ptr = puffer;
     errno=0;
     if(!ruhig)
     {
       while((r=write(1,ptr,rest))!=0 && rest > 0)
       {
	 if(r<0)
	 { switch(errno)
	   {
	   case EAGAIN: 
	   case EINTR:
	     write(2,"write: nochmal\n",sizeof("write: nochmal\n"));
	     errno=0;
	     continue;
           default:
	     break;
	   } 
	 }
	 ptr += r;
	 rest -= r;
       }
     } /* !ruhig */
   } 

   if(llang<0 && errno !=0)
   {   
     sprintf(errmsg,"read (%s:%d):",__FILE__,__LINE__);
     perror(errmsg);
   }
   gettimeofday(&ende,NULL);
   write(2,"Ende, schliee Sockets\n",sizeof("Ende, schliee Sockets\n"));
   /* sockets schliessen */
   close(sneu);
   close(s0);
   cleanupaddr(maddr);
   printstat(&anfang,&ende,receivedbytes);
}

void cleanupaddr(struct sockaddr *partner)
{
  if(partner->sa_family==AF_UNIX)
  {
    struct sockaddr_un *a = (struct sockaddr_un *)partner;
    unlink(a->sun_path);
  }
}





