/************************************************************************
 * Probeprogramm...: client.c						*
 * Zweck...........: Kundenprogramm, dass etwas an Server sendet	*
 ************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<sys/un.h> /* Unix Sockets */
#include<netdb.h>
#include"messung.h"

#define PUFLEN 4096
char puffer[PUFLEN];
char errmsg[256];


void printstat(struct timeval *prec,struct timeval *start,struct timeval *stop,long anz)
{
  double zeit = diff(start,stop);
  fprintf(stderr,"Latenz connect %g s \n",diff(prec,start));
  fprintf(stderr,"%ld bytes in %g s gesendet\n",anz,zeit);
  fprintf(stderr,"Durchsatz %g bytes/s\n",anz/zeit);
  zeit = diff(prec,stop);
  fprintf(stderr,"Gesamtzeit: %g\n",zeit);
  fprintf(stderr,"Gesamtzeitdurchsatz: %g\n",anz/zeit);
  
}

int init_sockaddr_host_port(struct sockaddr *partner,int *addrlen,char *host,short port)
{
  struct hostent *hp,*gethostbyname();
  struct sockaddr_in *addr=(struct sockaddr_in *)partner;
  *addrlen=sizeof(struct sockaddr_in);

  /* Adresse des Servers aus Kommandozeitenparameter ermitteln */
   addr->sin_family = AF_INET;
   hp = gethostbyname(host);
   if(hp == NULL)
   {
      fprintf(stderr,"gethostbyname(%s:%d): Rechner %s nicht gefunden\n",
					   __FILE__,__LINE__,host);
      return -1;
   }
   bcopy(hp->h_addr_list[0],&(addr->sin_addr),hp->h_length);
   addr->sin_port = htons(port);
   return 0;
}

int init_sockaddr(struct sockaddr *partner,int *addrlen,char *addresse)
{
  if(strncmp(addresse,"inet:",5)==0)
  {
    char *rechner=alloca(strlen(addresse));
    int port;
    if(sscanf(addresse+5,"%[^:]:%d",rechner,&port)==2)
    {
      return init_sockaddr_host_port(partner,addrlen,rechner,port);
    }
    else
    {
      fprintf(stderr,"Ungltige Adresse %s\n",addresse);
      return -1;
    }
  }
  else if(strncmp(addresse,"unix:",5)==0)
  {
    struct sockaddr_un *addr = (struct sockaddr_un *)partner;
    addr->sun_family = AF_UNIX;
    strcpy(addr->sun_path,addresse+5);
    *addrlen = SUN_LEN(addr);
    return 0;
  }
  else 
  {
    fprintf(stderr,"Unknown Protocol %s\n",addresse);
    return -1;
  }
}

int sender(struct sockaddr *partner,int addrlen,int v)
{
   int s;		/* Socket */
   struct hostent *hp,*gethostbyname();
   int slang;
   struct timeval prec,anfang,ende;
   long gesendet=0;

   if((s = socket(partner->sa_family,SOCK_STREAM,0))<0)
   {
      sprintf(errmsg,"socket(%s:%d):",__FILE__,__LINE__);
      perror(errmsg);
      exit(1);
   }

   gettimeofday(&prec,NULL);
   if(connect(s,partner,addrlen)<0)
   {
      sprintf(errmsg,"connect(%s:%d):",__FILE__,__LINE__);
      perror(errmsg);
      exit(1);
   }
   
   gettimeofday(&anfang,NULL);
   while((slang=read(0,puffer,PUFLEN))>0)
   {
       if(v) write(1,puffer,slang);
       if(write(s,puffer,slang)<0)
       {
	   sprintf(errmsg,"write(%s:%d):",__FILE__,__LINE__);
	   perror(errmsg);
       }
       gesendet+=slang;
   }
   gettimeofday(&ende,NULL);
   /*--- Ende ---- */
   close(s);
   printstat(&prec,&anfang,&ende,gesendet);
}

int main(int argc,char **argv)
{
  int v=0;
  int o;
  int addrlen;
  struct sockaddr partner;
  char sicherheit[64];
  memset((char *)&partner,'\0',sizeof(struct sockaddr));
  memset(sicherheit,'\0',64);
  while((o=getopt(argc,argv,"vs"))!=-1)
  {
    switch(o)
    {
      case 'v':
               v = 1;
               break;
    }
  }
  if(argc-optind == 2)
  {
    int port = atoi(argv[optind+1]);
    fprintf(stderr,"Sende an %s port %d\n",argv[optind],port);
    if(init_sockaddr_host_port(&partner,&addrlen,argv[optind],port)<0) exit(1);
    sender(&partner,addrlen,v);
  }
  else if(argc-optind == 1)
  {
    if(init_sockaddr(&partner,&addrlen,argv[optind])<0) exit(1);
    sender(&partner,addrlen,v);
  }
}



